# -*- coding: utf-8 -*-

import six

from wargaming import settings
from wargaming.api import BaseAPI, MetaAPI
from .clans import Clans

__all__ = ('API', )


@six.add_metaclass(MetaAPI)
class API(BaseAPI):
    """Wargaming.NET API client"""

    def __init__(self, application_id, language=settings.DEFAULT_LANGUAGE):
        """
        :param application_id: Your application ID from the https://wargaming.net/developers/applications/
        :type application_id: str
        :param language: Language for the requests (defaults to English)
        :type language: str
        """

        super(API, self).__init__(application_id, language,
                                  base_url='https://api.worldoftanks.com/wgn/')

    clans = Clans()



from wargaming.api import SubclassAPI, bind


class GlobalWar(SubclassAPI):

    clans = bind(path='globalwar/clans/',
        allowed_params=('fields', 'language', 'map_id', 'limit', 'page_no'),
        doc='''Clans

        :reference: https://wargaming.net/developers/api_reference/wot/globalwar/clans/''')

    fame_points = bind(path='globalwar/famepoints/',
        allowed_params=('fields', 'language', 'map_id', 'account_id'),
        doc='''Fame points

        :reference: https://na.wargaming.net/developers/api_reference/wot/globalwar/famepoints/''')

    maps = bind(path='globalwar/maps/',
        allowed_params=('fields', 'language'),
        doc='''Maps

        :reference: https://wargaming.net/developers/api_reference/wot/globalwar/maps/''')

    provinces = bind(path='globalwar/provinces/',
        allowed_params=('fields', 'language', 'map_id', 'province_id', 'region_id'),
        doc='''Provinces

        :reference: https://wargaming.net/developers/api_reference/wot/globalwar/provinces/''')

    tournaments = bind(path='globalwar/tournaments/',
        allowed_params=('fields', 'language', 'map_id', 'province_id'),
        doc='''Tournaments

        :reference: https://wargaming.net/developers/api_reference/wot/globalwar/tournaments/''')

    fame_points_history = bind(path='globalwar/famepointshistory/',
        allowed_params=('fields', 'language', 'map_id', 'since', 'until', 'page_no', 'limit', 'access_token'),
        doc='''Player's fame points log

        :reference: https://wargaming.net/developers/api_reference/wot/globalwar/famepointshistory/''')

    alley_of_fame = bind(path='globalwar/alleyoffame/',
        allowed_params=('fields', 'language', 'map_id', 'limit', 'page_no'),
        doc='''Alley of fame

        :reference: https://wargaming.net/developers/api_reference/wot/globalwar/alleyoffame/''')

    battles = bind(path='globalwar/battles/',
        allowed_params=('fields', 'language', 'map_id', 'clan_id', 'access_token'),
        doc='''Clan's battles

        :reference: https://wargaming.net/developers/api_reference/wot/globalwar/battles/''')

    victory_points_history = bind(path='globalwar/victorypointshistory/',
        allowed_params=('fields', 'language', 'map_id', 'clan_id', 'since', 'until', 'offset', 'limit'),
        doc='''Clan victory points log

        :rerefence: https://wargaming.net/developers/api_reference/wot/globalwar/victorypointshistory/''')



from wargaming.api import SubclassAPI, bind


class Tanks(SubclassAPI):

    stats = bind(path='tanks/stats/',
        allowed_params=('language', 'fields', 'access_token', 'account_id', 'tank_id', 'in_garage'),
        doc='''Vehicle statistics

        :reference: http://wargaming.net/developers/api_reference/wot/tanks/stats/''')

    achievements = bind(path='tanks/achievements/',
        allowed_params=('language', 'fields', 'access_token', 'account_id', 'tank_id', 'in_garage'),
        doc='''Vehicle achievements

        :reference: http://wargaming.net/developers/api_reference/wot/tanks/achievements/''')



from wargaming.version import get_version


HTTP_USER_AGENT_HEADER = 'python-wargaming/{0} (https://github.com/svartalf/python-wargaming)'.format(get_version())

DEFAULT_LANGUAGE = 'en'

ALLOWED_LANGUAGES = ('en', 'ru', 'pl', 'de', 'fr', 'es', 'zh-cn', 'tr', 'cs',
                     'th', 'vi', 'ko')



from wargaming.api import SubclassAPI, bind


class ClanRatings(SubclassAPI):

    types = bind(path='clanratings/types/',
        allowed_params=('fields', 'language'),
        doc='''Types of ratings

        :reference: http://wargaming.net/developers/api_reference/wot/clanratings/types/''')

    dates = bind(path='clanratings/dates/',
        allowed_params=('fields', 'language', 'type', 'clan_id'),
        doc='''Dates with available ratings

        :reference: http://wargaming.net/developers/api_reference/wot/clanratings/dates/''')

    clans = bind(path='clanratings/clans/',
        allowed_params=('fields', 'language', 'type', 'date', 'clan_id'),
        doc='''Clan ratings

        :reference: http://wargaming.net/developers/api_reference/wot/clanratings/clans/''')

    neighbors = bind(path='clanratings/neighbors/',
        allowed_params=('fields', 'language', 'type', 'date', 'clan_id', 'rank_field', 'limit'),
        doc='''Adjacent positions in clan rating

        :reference: http://wargaming.net/developers/api_reference/wot/clanratings/neighbors/''')

    top = bind(path='clanratings/top/',
        allowed_params=('fields', 'language', 'type', 'date', 'rank_field', 'limit'),
        doc='''Top clans

        :reference: http://wargaming.net/developers/api_reference/wot/clanratings/top/''')



# -*- coding: utf-8 -*-

import datetime
import warnings

import requests
import six
from six.moves.urllib.parse import urlparse, urlunparse, urljoin, urlencode

from wargaming import settings
from wargaming.exceptions import ValidationError, RequestError


class MetaAPI(type):

    def __new__(cls, name, bases, attrs):
        module = attrs.pop('__module__')

        new_cls = super(MetaAPI, cls).__new__(cls, name, bases, {
            '__module__': module,
            'subclasses': {},
        })

        for k, v in attrs.items():
            if not isinstance(v, SubclassAPI):
                setattr(new_cls, k, v)
            else:
                new_cls.subclasses[k] = v

        return new_cls


class BaseAPI(object):

    def __init__(self, application_id, language=settings.DEFAULT_LANGUAGE,
                 base_url=''):

        self.application_id = application_id
        self.language = language

        url_parts = urlparse(base_url)
        self.url_scheme = url_parts.scheme
        self.url_netloc = url_parts.netloc
        self.url_path = url_parts.path

        for name, subclass in self.subclasses.items():
            subclass.contribute_to_class(self)
            setattr(self, name, subclass)

    def _get_url(self, path, parameters):
        return urlunparse([
            self.url_scheme,
            self.url_netloc,
            urljoin(self.url_path, path),
            None,
            urlencode(parameters),
            None
        ])

    def _request(self, path, **kwargs):
        parameters = {}

        for key, value in kwargs.items():
            if value is None:
                continue

            prepare_func = getattr(self, 'prepare_{0}'.format(key), None)
            if prepare_func is not None:
                value = prepare_func(value)

            if isinstance(value, (list, tuple)):
                value = ','.join([str(x) for x in value])
            elif isinstance(value, bool):
                value = int(value)
            elif isinstance(value, (datetime.datetime, datetime.date)):
                value = value.isoformat()

            parameters[key] = value

        parameters.update({
            'application_id': self.application_id,
        })

        if 'language' not in parameters:
            parameters['language'] = self.language

        url = self._get_url(path, parameters)

        response = requests.get(url, headers={
            'User-Agent': settings.HTTP_USER_AGENT_HEADER,
        }).json()

        if response['status'] == 'error':
            raise RequestError(**response['error'])

        return response['data']

    @staticmethod
    def prepare_language(value):
        """Localization language

        One of the :data:`wargaming.settings.ALLOWED_LANGUAGES`
        """

        if value not in settings.ALLOWED_LANGUAGES:
            raise ValidationError('Invalid language: {0}'.format(value))

        return value


class SubclassAPI(object):
    """Base class for all of the API endpoint 'namespaces'"""

    def __init__(self):
        self.api = None

    def contribute_to_class(self, instance):
        self.api = instance


def bind(path, allowed_params=(), doc=None):
    """Make a API method callable for class inherited from :class:`SubclassAPI`

    :param path: API endpoint path, relative to game base url
    :type path: str
    :param allowed_params: Allowed parameters names
    :type allowed_params: list | tuple
    :param doc: docstring for method callable
    :type doc: str | unicode
    """

    def request(self, **kwargs):
        for param in kwargs.keys():
            if param not in allowed_params:
                raise ValidationError('Wrong parameter: {0}'.format(param))

        return self.api._request(path, **kwargs)

    method_name = path.strip('/').replace('/', '_').lower()
    request.__name__ = method_name

    if doc is not None:
        # Creating proper docstring based on the `doc` and `allowed_params`
        # Also creating a dumb usage example. Just for case.

        docstring = six.StringIO()
        for line in doc.strip().splitlines():
            docstring.write(line.strip())
            docstring.write('\n')
        docstring.write('\n')
        for param in allowed_params:
            docstring.write(':param {0}:\n'.format(param))

        request.__doc__ = docstring.getvalue()

    return request



from wargaming.api import SubclassAPI, bind


class Auth(SubclassAPI):

    login = bind(path='auth/login/',
        allowed_params=('fields', 'language', 'expires_at', 'redirect_uri', 'display', 'nofollow'),
        doc='''OpenID login

        :reference: https://wargaming.net/developers/api_reference/wot/auth/login/''')

    prolongate = bind(path='auth/prolongate/',
        allowed_params=('expires_at', 'access_token'),
        doc='''Access token extension
        :reference: https://wargaming.net/developers/api_reference/wot/auth/prolongate/''')

    logout = bind(path='auth/logout/',
        allowed_params=('access_token', ),
        doc='''Log out

        :reference: https://wargaming.net/developers/api_reference/wot/auth/logout/''')



from wargaming.games.wot import API as WoT
from wargaming.games.wgn import API as WGN
from wargaming.games.wotb import API as WoTB
from wargaming.exceptions import APIError, RequestError, ValidationError
from wargaming.version import get_version
from . import settings

__version__ = get_version()



from wargaming.api import SubclassAPI, bind


class Encyclopedia(SubclassAPI):

    tanks = bind(path='encyclopedia/tanks/',
        allowed_params=('fields', 'language'),
        doc='''List of vehicles

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/tanks/''')

    tank_info = bind(path='encyclopedia/tankinfo/',
        allowed_params=('fields', 'language', 'tank_id'),
        doc='''Vehicle details

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/tankinfo/''')

    tank_engines = bind(path='encyclopedia/tankengines/',
        allowed_params=('fields', 'language', 'module_id', 'nation'),
        doc='''Engines

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/tankengines/''')

    tank_turrets = bind(path='encyclopedia/tankturrets/',
        allowed_params=('fields', 'language', 'module_id', 'nation'),
        doc='''Turrets

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/tankturrets/''')

    tank_radios = bind(path='encyclopedia/tankradios/',
        allowed_params=('fields', 'language', 'module_id', 'nation'),
        doc='''Radios

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/tankradios/''')

    tank_suspensions = bind(path='encyclopedia/tankchassis/',
        allowed_params=('fields', 'language', 'module_id', 'nation'),
        doc='''Suspensions

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/tankchassis/''')

    tank_guns = bind(path='encyclopedia/tankguns/',
        allowed_params=('fields', 'language', 'module_id', 'nation', 'turret_id', 'tank_id'),
        doc='''

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/tankguns/''')

    achievements = bind(path='encyclopedia/achievements/',
        allowed_params=('fields', 'language'),
        doc='''Achievements

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/achievements/''')

    info = bind(path='encyclopedia/info/',
        allowed_params=('fields', 'language'),
        doc='''Tankopedia information

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/info/''')

    arenas = bind(path='encyclopedia/arenas/',
        allowed_params=('fields', 'language'),
        doc='''Maps details

        :reference: https://na.wargaming.net/developers/api_reference/wot/encyclopedia/arenas/''')

    vehicles = bind(path='encyclopedia/vehicles/',
        allowed_params=('fields', 'tank_id', 'nation', 'tier'),
        doc='''Vehicles

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/vehicles/''')

    vehicle_profile = bind(path='encyclopedia/vehicleprofile/',
        allowed_params=('fields', 'tank_id', 'engine_id', 'gun_id', 'suspension_id', 'turret_id', 'radio_id'),
        doc='''Vehicle characteristics

        :reference: https://wargaming.net/developers/api_reference/wot/encyclopedia/vehicleprofile/''')



# -*- coding: utf-8 -*-

import six

from wargaming import settings
from wargaming.api import BaseAPI, MetaAPI
from wargaming.games.wot.accounts import Accounts
from wargaming.games.wot.auth import Auth
from wargaming.games.wot.clans import Clans
from wargaming.games.wot.globalwar import GlobalWar
from wargaming.games.wot.encyclopedia import Encyclopedia
from wargaming.games.wot.ratings import Ratings
from wargaming.games.wot.clan_ratings import ClanRatings
from wargaming.games.wot.tanks import Tanks
from wargaming.games.wot.strongholds import Strongholds

__all__ = ('API', )


@six.add_metaclass(MetaAPI)
class API(BaseAPI):
    """World of Tanks API client"""

    def __init__(self, application_id, language=settings.DEFAULT_LANGUAGE):
        """
        :param application_id: Your application ID from the https://wargaming.net/developers/applications/
        :type application_id: str
        :param language: Language for the requests (defaults to English)
        :type language: str
        """

        super(API, self).__init__(application_id, language,
                                  base_url='https://api.worldoftanks.com/wot/')

    accounts = Accounts()

    auth = Auth()

    clans = Clans()

    globalwar = GlobalWar()

    encyclopedia = Encyclopedia()

    ratings = Ratings()

    clan_ratings = ClanRatings()

    tanks = Tanks()

    strongholds = Strongholds()



from wargaming.api import SubclassAPI, bind


class Accounts(SubclassAPI):
    """Players information, vehicles and achievements"""

    list = bind(path='account/list/',
        allowed_params=('fields', 'language', 'type', 'search', 'limit'),
        doc='''List of players

        :reference: https://wargaming.net/developers/api_reference/wotb/account/list/''')

    info = bind(path='account/info/',
        allowed_params=('fields', 'language', 'account_id', 'access_token'),
        doc='''Player personal data

        :reference: https://wargaming.net/developers/api_reference/wot/account/info/''')

    achievements = bind(path='account/achievements/',
        allowed_params=('fields', 'language', 'account_id'),
        doc='''Player's achievements

        :reference: https://wargaming.net/developers/api_reference/wotb/account/achievements/''')



VERSION = (0, 4, 1)


def get_version(version=None):
    """Derives a PEP386-compliant version number from VERSION."""

    if version is None:
        version = VERSION

    return '.'.join(str(x) for x in version)



from wargaming.api import SubclassAPI, bind


class Clans(SubclassAPI):

    provinces = bind(path='clan/provinces/',
        allowed_params=('fields', 'language', 'map_id', 'clan_id', 'access_token'),
        doc='''Clan's provinces

        :reference: https://wargaming.net/developers/api_reference/wot/clan/provinces/''')



from wargaming.api import SubclassAPI, bind


class Ratings(SubclassAPI):

    types = bind(path='ratings/types/',
        allowed_params=('language', 'fields'),
        doc='''Types of ratings

        :reference: https://wargaming.net/developers/api_reference/wot/ratings/types/''')

    dates = bind(path='ratings/dates/',
        allowed_params=('language', 'fields', 'type', 'account_id'),
        doc='''Dates with available ratings

        :reference: https://wargaming.net/developers/api_reference/wot/ratings/dates/''')

    accounts = bind(path='ratings/accounts/',
        allowed_params=('language', 'fields', 'type', 'date', 'account_id'),
        doc='''Player ratings

        :reference: https://wargaming.net/developers/api_reference/wot/ratings/accounts/''')

    neighbors = bind(path='ratings/neighbors/',
        allowed_params=('language', 'fields', 'type', 'date', 'account_id', 'rank_field', 'limit'),
        doc='''Adjacent positions in rating

        :reference: https://na.wargaming.net/developers/api_reference/wot/ratings/neighbors/''')

    top = bind(path='ratings/top/',
        allowed_params=('language', 'fields', 'type', 'date', 'account_id', 'rank_field', 'limit'),
        doc='''Top players

        :reference: https://na.wargaming.net/developers/api_reference/wot/ratings/top/''')



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        pytest.main(self.test_args)


def lines(filename):
    return [x.strip() for x in open(filename).read().splitlines()]

install_requires = lines('requirements.txt')

test_requires = lines('requirements_test.txt')

setup(
    name='wargaming',
    version='0.4.1',
    author='svartalf',
    author_email='self@svartalf.info',
    url='https://github.com/svartalf/python-wargaming',
    description='API library for Wargaming.net',
    long_description=__doc__,
    license='MIT',
    packages=find_packages(exclude=['tests', 'docs']),
    install_requires=install_requires,
    tests_require=install_requires + test_requires,
    cmdclass={'test': PyTest},
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries',
    ),
)



from wargaming.api import SubclassAPI, bind


class Clans(SubclassAPI):

    list = bind(path='clans/list/',
        allowed_params=('fields', 'language', 'search', 'limit', 'order_by', 'page_no'),
        doc='''Clans

        :reference: https://wargaming.net/developers/api_reference/wgn/clans/list/''')

    info = bind(path='clans/info/',
        allowed_params=('fields', 'language', 'clan_id', 'access_token'),
        doc='''Clan details

        :reference: https://wargaming.net/developers/api_reference/wgn/clans/info/''')

    membersinfo = bind(path='clans/membersinfo/',
        allowed_params=('fields', 'language', 'account_id'),
        doc='''Clan member

        :reference: https://wargaming.net/developers/api_reference/wgn/clans/membersinfo/''')



from wargaming.api import SubclassAPI, bind


class Strongholds(SubclassAPI):

    info = bind(path='stronghold/info/',
        allowed_params=('fields', 'language', 'clan_id', 'access_token'),
        doc='''Clan's stronghold

        :reference: https://wargaming.net/developers/api_reference/wot/stronghold/info/''')

    structures = bind(path='stronghold/buildings/',
        allowed_params=('fields', 'language'),
        doc='''Encyclopedia information on all structures of the Stronghold.

        :reference: https://wargaming.net/developers/api_reference/wot/stronghold/buildings/''')

    account_stats = bind('stronghold/accountstats/',
        allowed_params=('fields', 'language', 'access_token', 'account_id'),
        doc='''Player stats for the current clan's Stronghold..

        :reference: https://wargaming.net/developers/api_reference/wot/stronghold/accountstats/''')



from wargaming.api import SubclassAPI, bind


class Encyclopedia(SubclassAPI):

    vehicles = bind(path='encyclopedia/vehicles/',
        allowed_params=('fields', 'language', 'tank_id', 'nation'),
        doc='''List of vehicles

        :reference: https://wargaming.net/developers/api_reference/wotb/encyclopedia/vehicles/''')

    vehicle_profiles = bind(path='encyclopedia/vehicleprofiles/',
        allowed_params=('fields', 'language', 'tank_id', 'is_default'),
        doc='''Vehicle characteristics

        :reference: https://wargaming.net/developers/api_reference/wotb/encyclopedia/vehicleprofiles/''')

    modules = bind(path='encyclopedia/modules/',
        allowed_params=('fields', 'language', 'module_id'),
        doc='''Available modules, such as guns, engines, etc.

        :reference: https://wargaming.net/developers/api_reference/wotb/encyclopedia/modules/''')

    provisions = bind(path='encyclopedia/provisions/',
        allowed_params=('fields', 'language', 'type', 'provision_id', 'type_id'),
        doc='''Available equipment and consumables.

        :reference: https://wargaming.net/developers/api_reference/wotb/encyclopedia/provisions/''')

    info = bind(path='encyclopedia/info/',
        allowed_params=('fields', 'language'),
        doc='''Tankopedia information

        :reference: https://wargaming.net/developers/api_reference/wotb/encyclopedia/info/''')
